@@ -2,7 +2,7 @@
 psycopg cursor objects
 """
 
-# Copyright (C) 2020-2021 The Psycopg Team
+# Copyright (C) 2020 The Psycopg Team
 
 import sys
 from types import TracebackType
@@ -157,7 +157,7 @@ class BaseCursor(Generic[ConnectionType, Row]):
         methods `!fetch*()` will operate on.
         """
         if self._iresult < len(self._results) - 1:
-            self._set_result(self._iresult + 1)
+            self._set_current_result(self._iresult + 1)
             return True
         else:
             return None
@@ -197,8 +197,9 @@ class BaseCursor(Generic[ConnectionType, Row]):
         results = yield from self._maybe_prepare_gen(
             pgq, prepare=prepare, binary=binary
         )
-        self._set_results(results)
-        self._set_result(0)
+        self._check_results(results)
+        self._results = results
+        self._set_current_result(0)
         self._last_query = query
 
     def _executemany_gen(
@@ -217,16 +218,18 @@ class BaseCursor(Generic[ConnectionType, Row]):
                 pgq.dump(params)
 
             results = yield from self._maybe_prepare_gen(pgq, prepare=True)
-            self._set_results(results)
+            self._check_results(results)
 
             for res in results:
                 nrows += res.command_tuples or 0
 
-        self._set_result(0)
-        # Override rowcout for the first result. Calls to nextset() will change
+        if self._results:
+            self._set_current_result(0)
+
+        # Override rowcount for the first result. Calls to nextset() will change
         # it to the value of that result only, but we hope nobody will notice.
         # You haven't read this comment.
-        self._rowcount = nrows or -1
+        self._rowcount = nrows
         self._last_query = query
 
     def _maybe_prepare_gen(
@@ -305,7 +308,7 @@ class BaseCursor(Generic[ConnectionType, Row]):
 
         else:
             # Errors, unexpected values
-            return self._raise_from_results([res])
+            return self._raise_for_result(res)
 
     def _start_query(self, query: Optional[Query] = None) -> PQGen[None]:
         """Generator to start the processing of a query.
@@ -389,20 +392,39 @@ class BaseCursor(Generic[ConnectionType, Row]):
         ExecStatus.COPY_BOTH,
     )
 
-    def _set_results(self, results: List["PGresult"]) -> None:
+    def _check_results(self, results: List["PGresult"]) -> None:
         """
-        Set the results from a query into the cursor state.
+        Verify that the results of a query are valid.
+
+        Verify that the query returned at least one result and that they all
+        represent a valid result from the database.
         """
         if not results:
             raise e.InternalError("got no result from the query")
 
         for res in results:
             if res.status not in self._status_ok:
-                self._raise_from_results(results)
+                self._raise_for_result(res)
 
-        self._results = results
+    def _raise_for_result(self, result: "PGresult") -> NoReturn:
+        """
+        Raise an appropriate error message for an unexpected database result
+        """
+        if result.status == ExecStatus.FATAL_ERROR:
+            raise e.error_from_result(result, encoding=self._encoding)
+        elif result.status in self._status_copy:
+            raise e.ProgrammingError(
+                "COPY cannot be used with this method; use copy() insead"
+            )
+        else:
+            raise e.InternalError(
+                f"unexpected result status from query:"
+                f" {ExecStatus(result.status).name}"
+            )
 
-    def _set_result(self, i: int, format: Optional[Format] = None) -> None:
+    def _set_current_result(
+        self, i: int, format: Optional[Format] = None
+    ) -> None:
         """
         Select one of the results in the cursor as the active one.
         """
@@ -419,21 +441,6 @@ class BaseCursor(Generic[ConnectionType, Row]):
         nrows = self.pgresult.command_tuples
         self._rowcount = nrows if nrows is not None else -1
 
-    def _raise_from_results(self, results: List["PGresult"]) -> NoReturn:
-        statuses = {res.status for res in results}
-        badstats = statuses.difference(self._status_ok)
-        if results[-1].status == ExecStatus.FATAL_ERROR:
-            raise e.error_from_result(results[-1], encoding=self._encoding)
-        elif statuses.intersection(self._status_copy):
-            raise e.ProgrammingError(
-                "COPY cannot be used with this method; use copy() insead"
-            )
-        else:
-            raise e.InternalError(
-                f"got unexpected status from query:"
-                f" {', '.join(sorted(ExecStatus(s).name for s in badstats))}"
-            )
-
     def _send_prepare(self, name: bytes, query: PostgresQuery) -> None:
         self._pgconn.send_prepare(name, query.query, param_types=query.types)
 
@@ -449,7 +456,7 @@ class BaseCursor(Generic[ConnectionType, Row]):
             name, pgq.params, param_formats=pgq.formats, result_format=fmt
         )
 
-    def _check_result(self) -> None:
+    def _check_result_for_fetch(self) -> None:
         if self.closed:
             raise e.InterfaceError("the cursor is closed")
         res = self.pgresult
@@ -476,7 +483,7 @@ class BaseCursor(Generic[ConnectionType, Row]):
             )
 
     def _scroll(self, value: int, mode: str) -> None:
-        self._check_result()
+        self._check_result_for_fetch()
         assert self.pgresult
         if mode == "relative":
             newpos = self._pos + value
@@ -565,8 +572,11 @@ class Cursor(BaseCursor["Connection[Any]", Row]):
         """
         Execute the same command with a sequence of input data.
         """
-        with self._conn.lock:
-            self._conn.wait(self._executemany_gen(query, params_seq))
+        try:
+            with self._conn.lock:
+                self._conn.wait(self._executemany_gen(query, params_seq))
+        except e.Error as ex:
+            raise ex.with_traceback(None)
 
     def stream(
         self,
@@ -597,7 +607,7 @@ class Cursor(BaseCursor["Connection[Any]", Row]):
 
         :rtype: Optional[Row], with Row defined by `row_factory`
         """
-        self._check_result()
+        self._check_result_for_fetch()
         record = self._tx.load_row(self._pos, self._make_row)
         if record is not None:
             self._pos += 1
@@ -611,7 +621,7 @@ class Cursor(BaseCursor["Connection[Any]", Row]):
 
         :rtype: Sequence[Row], with Row defined by `row_factory`
         """
-        self._check_result()
+        self._check_result_for_fetch()
         assert self.pgresult
 
         if not size:
@@ -630,7 +640,7 @@ class Cursor(BaseCursor["Connection[Any]", Row]):
 
         :rtype: Sequence[Row], with Row defined by `row_factory`
         """
-        self._check_result()
+        self._check_result_for_fetch()
         assert self.pgresult
         records = self._tx.load_rows(
             self._pos, self.pgresult.ntuples, self._make_row
@@ -639,7 +649,7 @@ class Cursor(BaseCursor["Connection[Any]", Row]):
         return records
 
     def __iter__(self) -> Iterator[Row]:
-        self._check_result()
+        self._check_result_for_fetch()
 
         def load(pos: int) -> Optional[Row]:
             return self._tx.load_row(pos, self._make_row)
