@@ -6,10 +6,10 @@ the `pq` module instead, which is in charge of choosing the best
 implementation.
 """
 
-# Copyright (C) 2020-2021 The Psycopg Team
+# Copyright (C) 2020 The Psycopg Team
 
-import os
 import logging
+from os import getpid
 from weakref import ref
 from functools import partial
 
@@ -24,6 +24,9 @@ from .misc import PGnotify, ConninfoOption, PGresAttDesc
 from .misc import error_message, connection_summary
 from ._enums import Format, ExecStatus
 
+# Imported locally to call them from __del__ methods
+from ._pq_ctypes import PQclear, PQfinish, PQfreeCancel, PQstatus
+
 if TYPE_CHECKING:
     from . import abc
 
@@ -63,7 +66,6 @@ class PGconn:
     Python representation of a libpq connection.
     """
 
-    __module__ = "psycopg.pq"
     __slots__ = (
         "_pgconn_ptr",
         "notice_handler",
@@ -83,12 +85,12 @@ class PGconn:
         )
         impl.PQsetNoticeReceiver(pgconn_ptr, self._notice_receiver, None)
 
-        self._procpid = os.getpid()
+        self._procpid = getpid()
 
     def __del__(self) -> None:
         # Close the connection only if it was created in this process,
         # not if this object is being GC'd after fork.
-        if os.getpid() == self._procpid:
+        if getpid() == self._procpid:
             self.finish()
 
     def __repr__(self) -> str:
@@ -122,7 +124,7 @@ class PGconn:
     def finish(self) -> None:
         self._pgconn_ptr, p = None, self._pgconn_ptr
         if p:
-            impl.PQfinish(p)
+            PQfinish(p)
 
     @property
     def pgconn_ptr(self) -> Optional[int]:
@@ -202,7 +204,7 @@ class PGconn:
 
     @property
     def status(self) -> int:
-        return impl.PQstatus(self._pgconn_ptr)
+        return PQstatus(self._pgconn_ptr)
 
     @property
     def transaction_status(self) -> int:
@@ -713,7 +715,6 @@ class PGresult:
     Python representation of a libpq result.
     """
 
-    __module__ = "psycopg.pq"
     __slots__ = ("_pgresult_ptr",)
 
     def __init__(self, pgresult_ptr: impl.PGresult_struct):
@@ -730,7 +731,7 @@ class PGresult:
     def clear(self) -> None:
         self._pgresult_ptr, p = None, self._pgresult_ptr
         if p:
-            impl.PQclear(p)
+            PQclear(p)
 
     @property
     def pgresult_ptr(self) -> Optional[int]:
@@ -844,7 +845,6 @@ class PGcancel:
     Created by `PGconn.get_cancel()`.
     """
 
-    __module__ = "psycopg.pq"
     __slots__ = ("pgcancel_ptr",)
 
     def __init__(self, pgcancel_ptr: impl.PGcancel_struct):
@@ -863,7 +863,7 @@ class PGcancel:
         """
         self.pgcancel_ptr, p = None, self.pgcancel_ptr
         if p:
-            impl.PQfreeCancel(p)
+            PQfreeCancel(p)
 
     def cancel(self) -> None:
         """Requests that the server abandon processing of the current command.
@@ -885,8 +885,6 @@ class Conninfo:
     Utility object to manipulate connection strings.
     """
 
-    __module__ = "psycopg.pq"
-
     @classmethod
     def get_defaults(cls) -> List[ConninfoOption]:
         opts = impl.PQconndefaults()
@@ -940,8 +938,6 @@ class Escaping:
     Utility object to escape strings for SQL interpolation.
     """
 
-    __module__ = "psycopg.pq"
-
     def __init__(self, conn: Optional[PGconn] = None):
         self.conn = conn
 
